
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>

#include <util/delay.h>

#include "i2cmaster.h"
#include "i2clcd.h"

#define  LCD_CTRL_0  (1<<0)
#define  LCD_CTRL_1  (1<<1)

// use usbtiny library 
#include "usb.h"
#include "usbtiny.h"
typedef byte_t uchar;

#define USBDDR DDRD
#define USB_CFG_IOPORT PORTD

#define USB_CFG_DMINUS_BIT USBTINY_DMINUS
#define USB_CFG_DPLUS_BIT USBTINY_DPLUS

#define USBMASK (_BV(USBTINY_DPLUS) | _BV(USBTINY_DMINUS))

#define usbInit()  usb_init()
#define usbPoll()  usb_poll()


#define LED_DDR  DDRB						// Hardwarekonfiguration
#define LED_PORT PORTB
#define LED_PIN  PB1

#define ADDR_RTC 	0xA0
#define ADDR_LM75 	0x92

#define FROMBCD(x)      (((x) >> 4) * 10 + ((x) & 0xf))
#define TOBCD(x)        (((x) / 10 * 16) + ((x) % 10))

volatile unsigned char h_byte;	// Hight-Byte Temperatur-Registers LM75
volatile unsigned char l_byte;	// Low-Byte Temperatur-Registers LM75
volatile int click = 0;
volatile int lcd_intern = 1;


/* -------------------------------------------------------------------------- */
ISR(INT1_vect)
{
	click = 1;
//	LED_PORT ^= (1 << LED_PIN); 			// LED-PIN invertieren
}

/* -------------------------------------------------------------------------- */
void long_delay(uint16_t ms) {				// lange Pause
    for (; ms>0; ms--) _delay_ms(1);
}

/* -------------------------------------------------------------------------- */
unsigned char lm75_init(void)
{
	if (i2c_start(ADDR_LM75+I2C_WRITE)) return 1;
	i2c_write(0);							// Funktion Temperatur lesen
	i2c_stop();
	long_delay(100);						// etwas warten bevor es weiter geht...
	return 0;
}

/* -------------------------------------------------------------------------- */
unsigned char lm75_read(void)
{
	if (i2c_start(ADDR_LM75+I2C_READ)) return 1;
	h_byte = i2c_readAck();
	l_byte = i2c_readNak();
	i2c_stop();
	return 0;
}

/* -------------------------------------------------------------------------- */
unsigned char rtc_init(void)
{
	if (i2c_start(ADDR_RTC+I2C_WRITE)) return 1;
	i2c_write(0);					// Adresse 0
	i2c_write(0);					// Wert 0
	i2c_stop();
	long_delay(100);				// etwas warten bevor es weiter geht...
	return 0;
}

/* -------------------------------------------------------------------------- */
unsigned char rtc_read(unsigned char adr)
{
	unsigned char val; 

	if (i2c_start(ADDR_RTC+I2C_WRITE)) return 0;
	i2c_write(adr);					// zu lesende Adresse
	i2c_stop();
	long_delay(10);
	i2c_start(ADDR_RTC+I2C_READ);
	val = i2c_readNak();
	i2c_stop();
	return val;
}

/* -------------------------------------------------------------------------- */
unsigned char rtc_write(unsigned char adr, unsigned char val)
{
	if (i2c_start(ADDR_RTC+I2C_WRITE)) return 1;
	i2c_write(adr);					// Adresse
	i2c_write(val);					// Wert
	i2c_stop();
	return 0;
}


/* -------------------------------------------------------------------------- */
extern	byte_t	usb_setup ( byte_t data[8] )
{
	byte_t *replyBuf = data;
	uchar device = (data[1] >> 5);
	uchar len = (data[1] & 3)+1;       // 1 .. 4 bytes 
	uchar command = (data[1] >> 2) & 3; // command 0 .. 7

	/* command format:
	 * 7 6 5 4 3 2 1 0
	 * D D D C C C L L
	 *
	 * DDD = Device
	 * CCC = Command
	 * LL  = number of bytes in transfer - 1
	 */

	switch (device) {

	case 0: // blit_board
		switch (command) {

		case 0: // echo
			replyBuf[0] = data[2];
			replyBuf[1] = data[3];
			return 2;
			break;

		case 1: // set_led
			if (!data[2]) {
				LED_PORT &= ~(1 << LED_PIN); // off
			} else {
				LED_PORT |= (1 << LED_PIN);  // on
			}
		    break;

		default:
			break; 

		}
		break;

	case 1: // LCD_Board
		switch (command) {
		//case 0: // lcd_status
		//	break;

		case 1: // lcd_intern
			lcd_command(LCD_CLEAR);
			if (!data[2]) {
				lcd_intern = 0;
			} else {
				lcd_intern = 1;
			}
			break;
		
		case 2: // lcd_clear
			if (!lcd_intern) lcd_command(LCD_CLEAR);
			break;

		case 3: // lcd_backlight
			if (!lcd_intern) {
				if (!data[2]) {
					lcd_backlight(OFF);
				} else {
					lcd_backlight(ON);
				}
			lcd_busy();	// "Dummy-Befehl" damit etwas ueber den TWI-Bus gesendet wird und 
					// der entsprechende Port des PCF8574 geschaltet wird...
			}
			break;

		case 0: // lcd_out_text
			if (!lcd_intern) {
				// xy-Koordinate steht in data[3]: Bit 7...5 -> y, Bit 4...0 -> x
				// das anzuzeigende Zeichen in data[2]
				lcd_putcharlr(((data[3] & 0xe0) >> 5), (data[3] & 0x1f), data[2]);
			}
			break;

		default:
			break;
		}
		break;

	case 2: // LM75_Board
    		switch(command) {
		case 0: // lm75_status
			replyBuf[0] = lm75_init();
			replyBuf[1] = 0;
			return 2;
			break;

	  	case 1: // lm75_read
			lm75_read();
			replyBuf[0] = h_byte;
			replyBuf[1] = l_byte;
			return 2;
			break;

		default:
			break; 
     
		}
		break;

	case 3: // RTC_Board
    	switch(command) {
		case 0: // rtc_status
			replyBuf[0] = rtc_init();
			replyBuf[1] = 0;
			return 2;
			break;

	    case 1: // rtc_read
			replyBuf[0] = rtc_read(data[2]);
			return 1;
			break;

	    case 2: // rtc_write
			replyBuf[0] = rtc_write(data[2], data[3]);
			return 1;
			break;

		default:
			break; 
		}
		break;


	default:
	    break;
	}

  return 0;  // reply len
}

/* ------------------------------------------------------------------------- */
void lcd_putbcd_yx(unsigned char y, unsigned char x, unsigned char val) {
	lcd_putcharlr(y, x+1, (val & 0x0f) + '0');		
	lcd_putcharlr(y, x, ((val & 0xf0) >> 4) + '0');
} 



/* ------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------- */
int	main(void) {

	char s[10];
	unsigned char x, y;


	LED_DDR   = (1 << LED_PIN);		// LED-Pin als Ausgang schalten
	LED_PORT &= ~(1 << LED_PIN);

	MCUCR |= (1<<ISC10) | (1<<ISC11);	// steigende Flanke (L->H) fuer INT1
	GICR  |= (1<<INT1);          		// externer Interrupt an INT1

	usbInit();				// USB initialisieren
	i2c_init();				// TWI initialisieren
	lm75_init();				// Temperaturfuehler (LM75) initialisieren	
	rtc_init();				// RTC initialisieren
	lcd_init();				// LCD initialisieren

	sei();

	lcd_command(LCD_CLEAR);
	lcd_backlight(ON);
	lcd_printlc(1, 1, (unsigned char *)"-->BLIT-Board<--");
	lcd_printlc(2, 1, (unsigned char *)"Uwe Berger; 2008");
	lcd_wait_ms(1000);
	lcd_command(LCD_CLEAR);


	for(;;) {	
		usbPoll();
		
		if ((click) && (lcd_intern)) {			// INT1 aufgerufen?
			click = 0;
			lcd_printlc(2, 1, (unsigned char *)"Temp..: ");
			// RTC auslesen/aufbereiten
			lcd_putbcd_yx(1, 1, rtc_read(5));	// Tag
			lcd_putcharlr(1, 3, '.');
			lcd_putbcd_yx(1, 4, rtc_read(6));	// Monat
			lcd_putcharlr(1, 6, '.');
			lcd_putbcd_yx(1, 8, rtc_read(4));	// Stunde
			lcd_putcharlr(1, 10, ':');
			lcd_putbcd_yx(1, 11, rtc_read(3));	// Minute
			lcd_putcharlr(1, 13, ':');
			lcd_putbcd_yx(1, 14, rtc_read(2));	// Sekunde

			// LM75 auslesen/aufbereiten
			lm75_read();
			y = 2;
			x = 9;
			if (h_byte >= 128) {			// wenn Bit7 gesetzt, dann negative Temp.
				h_byte = ~(h_byte);		// dann (lt. Datenblatt) Byte invertieren
				h_byte++;			// und plus 1
				lcd_putcharlr(y, x, '-');
			} else {
				lcd_putcharlr(y, x, '+');
			}
			x++;
			itoa(h_byte, s, 10 );			// hier steht jetzt der Vorkomma-Wert
			lcd_printlr(y, x, (unsigned char *)s);
			x++;
			if (h_byte >= 10) x++;
			if (h_byte >= 100) x++;
			lcd_putcharlr(y, x, ',');
			x++;
			if (l_byte >= 128) {			// wenn Bit7 gesetzt, dann ,5 sonst ,0
				lcd_putcharlr(y, x, '5');
			} else {
				lcd_putcharlr(y, x, '0');
			}
			x++;
			lcd_putcharlr(y, x, 223);
			x++;
			lcd_putcharlr(y, x, 'C');
		}


	}
	return 0;
}

/* ------------------------------------------------------------------------- */
